/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init */

const Main = imports.ui.main;
const { FdoNotificationDaemonSource } = imports.ui.notificationDaemon;

const originalMethods = new Map();

function saveMethod(prototype, methodName) {
    originalMethods.set(methodName, prototype[methodName]);
}

function forEachSource(callback) {
    [
        FdoNotificationDaemonSource.prototype,
        ...Main.notificationDaemon._fdoNotificationDaemon._sources,
    ].forEach(callback);
}

function enable() {
    const fdoSourceProto = FdoNotificationDaemonSource.prototype;
    saveMethod(fdoSourceProto, 'destroyNonResidentNotifications');
    saveMethod(fdoSourceProto, 'openApp');
    
    const newDestroyNonResidentNotifications = function() {
        if (!this._appOpened)
            return;
        originalMethods.get('destroyNonResidentNotifications').call(this);
    }
    const newOpenApp = function() {
        this._appOpened = true;
        originalMethods.get('openApp').call(this);
        delete this._appOpened;
    }
    
    forEachSource(obj => {
        obj['destroyNonResidentNotifications'] = newDestroyNonResidentNotifications;
        obj['openApp'] = newOpenApp;
    });
}

function disable() {
    forEachSource(obj => {
        for ([name, method] of originalMethods)
            obj[name] = method;
    });
    originalMethods.clear();
}
